@@ -26,6 +26,7 @@
 #include "config.h"
 #include "hooks/hook.h"
 
+#include <bus/listeners/file_logger.h>
 #include <threading/thread.h>
 #include <credentials/certificates/x509.h>
 
@@ -80,7 +81,7 @@ static bool load_configs(char *suite_file, char *test_file)
 	}
 	conftest->test = settings_create(suite_file);
 	conftest->test->load_files(conftest->test, test_file, TRUE);
-	conftest->suite_dir = strdup(dirname(suite_file));
+	conftest->suite_dir = path_dirname(suite_file);
 	return TRUE;
 }
 
@@ -128,7 +129,7 @@ static bool load_cert(settings_t *settings, bool trusted)
 }
 
 /**
- * Load certificates from the confiuguration file
+ * Load certificates from the configuration file
  */
 static bool load_certs(settings_t *settings, char *dir)
 {
@@ -162,7 +163,7 @@ static bool load_certs(settings_t *settings, char *dir)
 }
 
 /**
- * Load private keys from the confiuguration file
+ * Load private keys from the configuration file
  */
 static bool load_keys(settings_t *settings, char *dir)
 {
@@ -289,7 +290,8 @@ static bool load_hooks()
 		pos = strchr(name, '-');
 		if (pos)
 		{
-			snprintf(buf, sizeof(buf), "%.*s_hook_create", pos - name, name);
+			snprintf(buf, sizeof(buf), "%.*s_hook_create", (int)(pos - name),
+					 name);
 		}
 		else
 		{
@@ -321,6 +323,7 @@ static bool load_hooks()
  */
 static void cleanup()
 {
+	file_logger_t *logger;
 	hook_t *hook;
 
 	DESTROY_IF(conftest->test);
@@ -343,10 +346,16 @@ static void cleanup()
 		}
 		conftest->config->destroy(conftest->config);
 	}
+	while (conftest->loggers->remove_last(conftest->loggers,
+										  (void**)&logger) == SUCCESS)
+	{
+		charon->bus->remove_logger(charon->bus, &logger->logger);
+		logger->destroy(logger);
+	}
+	conftest->loggers->destroy(conftest->loggers);
 	free(conftest->suite_dir);
 	free(conftest);
 	libcharon_deinit();
-	libhydra_deinit();
 	library_deinit();
 }
 
@@ -367,6 +376,24 @@ static void load_log_levels(file_logger_t *logger, char *section)
 	}
 }
 
+/**
+ * Load logger options for a logger from section
+ */
+static void load_logger_options(file_logger_t *logger, char *section)
+{
+	char *time_format;
+	bool add_ms, ike_name;
+
+	time_format = conftest->test->get_str(conftest->test,
+					"log.%s.time_format", NULL, section);
+	add_ms = conftest->test->get_bool(conftest->test,
+					"log.%s.time_add_ms", FALSE, section);
+	ike_name = conftest->test->get_bool(conftest->test,
+					"log.%s.ike_name", FALSE, section);
+
+	logger->set_options(logger, time_format, add_ms, ike_name);
+}
+
 /**
  * Load logger configuration
  */
@@ -374,26 +401,24 @@ static void load_loggers(file_logger_t *logger)
 {
 	enumerator_t *enumerator;
 	char *section;
-	FILE *file;
 
 	load_log_levels(logger, "stdout");
+	load_logger_options(logger, "stdout");
+	/* Re-add the logger to propagate configuration changes to the
+	 * logging system */
+	charon->bus->add_logger(charon->bus, &logger->logger);
 
 	enumerator = conftest->test->create_section_enumerator(conftest->test, "log");
 	while (enumerator->enumerate(enumerator, &section))
 	{
 		if (!streq(section, "stdout"))
 		{
-			file = fopen(section, "w");
-			if (file == NULL)
-			{
-				fprintf(stderr, "opening file %s for logging failed: %s",
-						section, strerror(errno));
-				continue;
-			}
-			logger = file_logger_create(file, NULL, FALSE);
+			logger = file_logger_create(section);
+			load_logger_options(logger, section);
+			logger->open(logger, FALSE, FALSE);
 			load_log_levels(logger, section);
-			charon->bus->add_listener(charon->bus, &logger->listener);
-			charon->file_loggers->insert_last(charon->file_loggers, logger);
+			charon->bus->add_logger(charon->bus, &logger->logger);
+			conftest->loggers->insert_last(conftest->loggers, logger);
 		}
 	}
 	enumerator->destroy(enumerator);
@@ -408,40 +433,35 @@ int main(int argc, char *argv[])
 	int status = 0;
 	sigset_t set;
 	int sig;
-	char *suite_file = "suite.conf", *test_file = NULL;
+	char *suite_file = "suite.conf", *test_file = NULL, *preload, *plugins;
 	file_logger_t *logger;
 
-	if (!library_init(NULL))
+	if (!library_init(NULL, "conftest"))
 	{
 		library_deinit();
 		return SS_RC_LIBSTRONGSWAN_INTEGRITY;
 	}
-	if (!libhydra_init("conftest"))
-	{
-		libhydra_deinit();
-		library_deinit();
-		return SS_RC_INITIALIZATION_FAILED;
-	}
 	if (!libcharon_init())
 	{
 		libcharon_deinit();
-		libhydra_deinit();
 		library_deinit();
 		return SS_RC_INITIALIZATION_FAILED;
 	}
 
 	INIT(conftest,
 		.creds = mem_cred_create(),
+		.config = config_create(),
+		.hooks = linked_list_create(),
+		.loggers = linked_list_create(),
 	);
-
-	logger = file_logger_create(stdout, NULL, FALSE);
-	logger->set_level(logger, DBG_ANY, LEVEL_CTRL);
-	charon->bus->add_listener(charon->bus, &logger->listener);
-	charon->file_loggers->insert_last(charon->file_loggers, logger);
-
 	lib->credmgr->add_set(lib->credmgr, &conftest->creds->set);
-	conftest->hooks = linked_list_create();
-	conftest->config = config_create();
+
+	logger = file_logger_create("stdout");
+	logger->set_options(logger, NULL, FALSE, FALSE);
+	logger->open(logger, FALSE, FALSE);
+	logger->set_level(logger, DBG_ANY, LEVEL_CTRL);
+	charon->bus->add_logger(charon->bus, &logger->logger);
+	conftest->loggers->insert_last(conftest->loggers, logger);
 
 	atexit(cleanup);
 
@@ -483,15 +503,19 @@ int main(int argc, char *argv[])
 	}
 	load_loggers(logger);
 
-	if (!lib->plugins->load(lib->plugins, NULL,
-			conftest->test->get_str(conftest->test, "preload", "")))
+	preload = conftest->test->get_str(conftest->test, "preload", "");
+	if (asprintf(&plugins, "%s %s", preload, PLUGINS) < 0)
 	{
 		return 1;
 	}
-	if (!charon->initialize(charon))
+	if (!charon->initialize(charon, plugins))
 	{
+		free(plugins);
 		return 1;
 	}
+	lib->plugins->status(lib->plugins, LEVEL_CTRL);
+	free(plugins);
+
 	if (!load_certs(conftest->test, conftest->suite_dir))
 	{
 		return 1;
@@ -533,7 +557,7 @@ int main(int argc, char *argv[])
 	sigaddset(&set, SIGTERM);
 	sigprocmask(SIG_BLOCK, &set, NULL);
 
-	while (sigwait(&set, &sig) == 0)
+	while ((sig = sigwaitinfo(&set, NULL)) != -1 || errno == EINTR)
 	{
 		switch (sig)
 		{
